<?php

require_once SMSAPI_PLUGIN_PATH . '/forms.php';

function sendSubscriptionWelcomeMessage($phonenumber)
{
    $notificationsSettings = get_option('smsapi_notifications', array());

    $welcomeNotificationEnabled = array_safe_get($notificationsSettings, 'subscription_notification');

    if ($welcomeNotificationEnabled) {
        $smsapiClient = new SMSApiClientProxy();

        $notificationContent = array_safe_get($notificationsSettings, 'subscription_notification_content');

        if ($notificationContent) {
            $smsapiClient->sendSms($phonenumber, $notificationContent);
        }
    }
}

function updateSubscriber(array $data)
{
    $smsapiClient = new SMSApiClientProxy();
    $phonenumber = array_safe_get($data, 'phonenumber');

    if ($subscriber = $smsapiClient->getSubscriberFromAnyGroup($phonenumber)) {
        $smsapiClient->overwriteSubscriber($subscriber, $data);
        $smsapiClient->bindSubscriberToDefaultGroup($subscriber);

        sendSubscriptionWelcomeMessage($phonenumber);
    }
}

function saveSubscriber(array $data)
{
    try {
        $smsapiClient = new SMSApiClientProxy();
        $subscriberPhoneNumber = array_safe_get($data, 'phonenumber');

        $smsapiClient->saveSubscriber($data);

        sendSubscriptionWelcomeMessage($subscriberPhoneNumber);

    } catch (\SMSApi\Exception\ActionException $error) {
        $contactsError = $error->getPrevious();

        if ($contactsError instanceof \SMSApi\Exception\ContactsException) {
            $contactConflictHttpStatus = 409;

            if ($contactsError->getCode() === $contactConflictHttpStatus) {
                updateSubscriber($data);
                return;
            }
        }

        throw $error;
    }
}

function registerSubscriber($formData) 
{
    $form = new SubscriptionForm($formData);

    if ($form->isValid()) {
        try {
            $subscriber = $form->getData();
            $subscriptionSettings = get_option('smsapi_subscription');

            if (array_safe_get($subscriptionSettings, 'subscription_security')) {
                continueRegistrationWithSecureCtx($subscriber);
            } else {
                saveSubscriber($subscriber);
                $response = array('message' => __('Subscription success', 'smsapi'));
            }
        } catch (\SMSApi\Exception\ActionException $error) {
            $response = array('message' => __('Something goes wrong. Please try again later.', 'smsapi'));
        }
    } else {
        $response = array('errors' => $form->getErrors());
    }

    echo json_encode($response);
}

function continueRegistrationWithSecureCtx(array $subscriber)
{
    $smsapiClient = new SMSApiClientProxy();

    $subscriberPhoneNumber = array_safe_get($subscriber, 'phonenumber');

    session_set('subscriber_to_save', $subscriber);
    session_set('secure_action', 'secure_subscribe');

    $securityCode = security_session_token(true);
    $message = sprintf("%s: %s", __("Your activation code is", 'smsapi'), $securityCode);

    $send = $smsapiClient->sendSms($subscriberPhoneNumber, $message);

    if ($send) {
        $response = array(
            'security_state' => true,
            'message' => __('Activation code was send to Your phonenumber.', 'smsapi')
        );
    } else {
        $response = array('message' => __('Something goes wrong. Please try again later.', 'smsapi'));
    }

    echo json_encode($response);
    exit;
}

function secureRegisterSubscriber()
{
    $securityToken = security_session_token();

    $form = new SecurityCodeForm($_POST, $securityToken);

    if ($form->isValid()) {
        $subscriber = session_get_once('subscriber_to_save');

        if (!$subscriber) {
            $response = array(
                'reset_form' => true,
                'message' => __('Something goes wrong. Please try again later.', 'smsapi')
            );
        } else {
            try {
                saveSubscriber($subscriber);

                $response = array(
                    'reset_form' => true,
                    'message' => __('Subscription success', 'smsapi')
                );
            } catch (\SMSApi\Exception\SmsapiException $error) {
                $response = array('message' => __('Something goes wrong. Please try again later.', 'smsapi'));
            }
        }
    } else {
        $response = array('errors' => $form->getErrors());
    }

    echo json_encode($response);
}

function unregisterSubscriber($formData) 
{
    $form = new UnsubscribeForm($formData);

    if ($form->isValid()) {
        try {
            $data = $form->getData();
            $smsapiClient = new SMSApiClientProxy();

            $subscriberPhoneNumber = array_safe_get($data, 'phonenumber');
            $subscriptionSettings = get_option('smsapi_subscription');

            if (array_safe_get($subscriptionSettings, 'subscription_security')) {
                continueUnregisterWithSecureCtx($subscriberPhoneNumber);
            } else {
                $subscriber = $smsapiClient->getSubscriber($subscriberPhoneNumber);

                if ($subscriber) {
                    $smsapiClient->removeSubscriber($subscriber->getId());
                }

                $response = array('message' => __('Phone number has been removed', 'smsapi'));
            }
        } catch (\SMSApi\Exception\SmsapiException $error) {
            $response = array('message' => __('Something goes wrong. Please try again later.', 'smsapi'));
        }
    } else {
        $response = array('errors' => $form->getErrors());
    }

    echo json_encode($response);
}

function continueUnregisterWithSecureCtx($subscriberPhoneNumber)
{
    $smsapiClient = new SMSApiClientProxy();

    $securityToken = security_session_token(true);

    session_set('phonenumber_to_remove', $subscriberPhoneNumber);
    session_set('secure_action', 'secure_unsubscribe');

    $message = sprintf("%s: %s", __('Your unregister code is', 'smsapi'), $securityToken);

    $send = $smsapiClient->sendSms($subscriberPhoneNumber, $message);

    if ($send) {
        $response = array(
            'security_state' => true,
            'message' => __('Unregister code was send to Your phonenumber.', 'smsapi')
        );
    } else {
        $response = array(
            'message' => __('Something goes wrong. Please try again later.', 'smsapi')
        );
    }

    echo json_encode($response);

    exit;
}

function secureUnregisterSubscriber()
{
    $securityToken = security_session_token();
    $phonenumber = session_get_once('phonenumber_to_remove');

    $form = new SecurityCodeForm($_POST, $securityToken);

    if ($form->isValid()) {
        try {
            $smsapiClient = new SMSApiClientProxy();

            $subscriber = $smsapiClient->getSubscriber($phonenumber);

            if (!$subscriber) {
                $response = array(
                    'reset_form' => true,
                    'message' => __('Something goes wrong. Please try again later.', 'smsapi')
                );
            } else {
                $smsapiClient->removeSubscriber($subscriber->getId());

                $response = array(
                    'reset_form' => true,
                    'message' => __('Phone number has been removed', 'smsapi')
                );
            }
        } catch (\SMSApi\Exception\ContactsException $error) {
            $response = array('message' => __('Something goes wrong. Please try again later.', 'smsapi'));
        }
    } else {
        $response = array('errors' => $form->getErrors());
    }

    echo json_encode($response);
}